"""Methods for dealing with OCF lab hours.

For simplicity, we assume each day has one set of hours, and that they are
contiguous (e.g. 9am-5pm). We don't support breaks in the middle or other weird
hours.

All times are assumed to be OST (OCF Standard Time).
"""
from collections import defaultdict
from collections import namedtuple
from datetime import date
from datetime import datetime

MONDAY = 0
TUESDAY = 1
WEDNESDAY = 2
THURSDAY = 3
FRIDAY = 4
SATURDAY = 5
SUNDAY = 6
REGULAR_HOURS = defaultdict(lambda: (9, 18), {
    SUNDAY: (12, 17),
    SATURDAY: (11, 18),
})
HOLIDAYS = {
    # start date, end date, holiday name, hours (date ranges are inclusive)
    (date(2015, 8, 1), date(2015, 8, 25), 'Summer Break', (None, None)),
    (date(2015, 9, 7), date(2015, 9, 7), 'Labor Day', (None, None)),
    (date(2015, 11, 11), date(2015, 11, 11), 'Veteran\'s Day', (None, None)),
    (date(2015, 11, 24), date(2015, 11, 24), 'Thanksgiving Break', (9, 12)),
    (date(2015, 11, 25), date(2015, 11, 27), 'Thanksgiving Break', (None, None)),
    (date(2015, 12, 7), date(2015, 12, 13), 'R.R.R. Week', (11, 21)),
    (date(2015, 12, 14), date(2015, 12, 17), 'Finals Week', (9, 21)),
    (date(2015, 12, 18), date(2015, 12, 18), 'Last Day Fall 2015', (9, 12)),
}


class DayHours(namedtuple('DayHours', ['name', 'holiday', 'open', 'close'])):

    @classmethod
    def from_date(cls, when=None):
        if not when:
            when = date.today()

        if isinstance(when, datetime):
            when = when.date()

        my_name = when.strftime('%A')  # e.g. 'Thursday'
        my_holiday = None
        my_hours = REGULAR_HOURS[when.weekday()]

        for start, end, name, hours in HOLIDAYS:
            if start <= when <= end:
                my_holiday = name
                my_hours = hours
                break

        return cls(
            name=my_name,
            holiday=my_holiday,
            open=my_hours[0],
            close=my_hours[1],
        )

    def is_open(self, when=None):
        if not when:
            when = datetime.now()

        if not isinstance(when, datetime):
            raise ValueError('{} must be a datetime instance'.format(when))

        if None in [self.open, self.close]:
            return False

        return self.open <= when.hour < self.close


def get_hours(when=None):
    """Return a DayHours object representing the day's hours.

    >>> get_hours()
    DayHours('Thursday', 9, 18)

    >>> get_hours()
    DayHours('Thursday', None, None)
    """
    return DayHours.from_date(when)


def is_open(when=None):
    """Returns a boolean whether the lab is currently open."""
    return get_hours(when).is_open(when)



from datetime import date
from datetime import datetime

import mock
import pytest
from freezegun import freeze_time

from ocflib.lab.hours import DayHours
from ocflib.lab.hours import FRIDAY
from ocflib.lab.hours import get_hours
from ocflib.lab.hours import is_open
from ocflib.lab.hours import MONDAY
from ocflib.lab.hours import REGULAR_HOURS
from ocflib.lab.hours import SATURDAY
from ocflib.lab.hours import SUNDAY
from ocflib.lab.hours import THURSDAY
from ocflib.lab.hours import TUESDAY
from ocflib.lab.hours import WEDNESDAY


FAKE_HOLIDAYS = {
    (date(2015, 3, 14), date(2015, 3, 14), 'Pi Day', (None, None)),
    (date(2015, 3, 20), date(2015, 3, 22), 'Random 3 Days', (1, 2)),
}
FAKE_REGULAR_HOURS = {
    0: (9, 18),  # Monday
    1: (9, 18),  # Tuesday
    2: (9, 18),  # Wednesday
    3: (9, 18),  # Thursday
    4: (9, 18),  # Friday
    5: (11, 18),  # Saturday
    6: (12, 17),  # Sunday
}


@pytest.yield_fixture
def mock_hours():
    with mock.patch('ocflib.lab.hours.HOLIDAYS', FAKE_HOLIDAYS), \
            mock.patch('ocflib.lab.hours.REGULAR_HOURS', FAKE_REGULAR_HOURS):
        yield FAKE_HOLIDAYS, FAKE_REGULAR_HOURS


@pytest.yield_fixture
def mock_today():
    with freeze_time('2015-08-22 14:11:00'):
        yield


@pytest.mark.parametrize('now,expected_open', [
    # regular hours
    (datetime(2015, 3, 15, 12), True),
    (datetime(2015, 3, 19, 17), True),
    (datetime(2015, 3, 19, 9), True),
    (datetime(2015, 3, 19, 18), False),
    (datetime(2015, 3, 19, 0), False),
    (None, True),

    # holidays
    (datetime(2015, 3, 14, 12), False),
    (datetime(2015, 3, 14, 0), False),
    (datetime(2015, 3, 20, 0), False),
    (datetime(2015, 3, 20, 1), True),
    (datetime(2015, 3, 20, 1, 30), True),
    (datetime(2015, 3, 20, 1, 59), True),
    (datetime(2015, 3, 20, 2, 0), False),
])
def test_is_open(now, expected_open, mock_hours, mock_today):
    assert is_open(now) == expected_open


def test_is_open_fails_with_just_date():
    with pytest.raises(ValueError):
        is_open(date(2015, 3, 14))


class TestDayHours:

    @pytest.mark.parametrize('when,name,holiday,open,close', [
        (date(2015, 3, 15), 'Sunday', None, 12, 17),
        (datetime(2015, 3, 15), 'Sunday', None, 12, 17),
        (datetime(2015, 3, 18), 'Wednesday', None, 9, 18),
        (datetime(2015, 3, 14), 'Saturday', 'Pi Day', None, None),
        (date(2015, 3, 22), 'Sunday', 'Random 3 Days', 1, 2),
        (None, 'Saturday', None, 11, 18),
    ])
    def test_creation(self, mock_hours, mock_today, when, name, holiday, open, close):
        for day_hours in [DayHours.from_date(when), get_hours(when)]:
            assert day_hours.name == name
            assert day_hours.open == open
            assert day_hours.holiday == holiday
            assert day_hours.close == close


def test_hours():
    for day in (SUNDAY, MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY):
        open, close = REGULAR_HOURS[day]
        assert isinstance(open, int)
        assert isinstance(close, int)
